/**
 * Copyright (C) : INRIA - Domaine de Voluceau, Rocquencourt, B.P. 105, 
 * 78153 Le Chesnay Cedex - France 
 * 
 * contributor(s) : SARDES project - http://sardes.inrialpes.fr
 *
 * Contact : jakub.kornas@inrialpes.fr
 *
 * This software is a computer program whose purpose is to provide a framework
 * to build autonomic systems, following an architecture-based approach.
 *
 * This software is governed by the CeCILL-C license under French law and 
 * abiding by the rules of distribution of free software. You can use, modify
 * and/or redistribute the software under the terms of the CeCILL-C license as 
 * circulated by CEA, CNRS and INRIA at the following URL 
 * "http://www.cecill.info". 
 *
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a 
 * limited warranty and the software's author, the holder of the economic 
 * rights, and the successive licensors have only limited liability. 
 *
 * In this respect, the user's attention is drawn to the risks associated with 
 * loading,  using,  modifying and/or developing or reproducing the software by 
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate,  and  that  also therefore means  that it is
 * reserved for developers  and  experienced professionals having in-depth 
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling 
 * the security of their systems and/or data to be ensured and,  more generally,
 * to use and operate it in the same conditions as regards security. 
 *
 * The fact that you are presently reading this means that you have had 
 * knowledge of the CeCILL-C license and that you accept its terms.
 */

package org.objectweb.fractal.deployment.local.api;

import java.util.List;

import org.objectweb.fractal.api.Component;

/**
 * Represents a locally installed {@link Component} package, its dependencies
 * and an associated {@link Loader}. A {@link LocallyInstalledPackage} object
 * is the result of installing by an {@link Installer} on a local machine the
 * resources obtained from the {@link PackageRepository} given a
 * {@link PackageDescription}.
 * 
 * For carbage collection purpose each {@link LocallyInstalledPackage} needs to
 * know how many other {@link LocallyInstalledPackage}s depend on it (even
 * though it does not need to know who exactly depends on it). Therefore it
 * maintains a reference counter.
 * 
 * @author <a href="mailto:jakub.kornas@inrialpes.fr">Jakub Kornas
 */
public interface LocallyInstalledPackage {

	/**
	 * Returns the {@link Loader} associated with this
	 * {@link LocallyInstalledPackage}. This {@link Loader} is able to load
	 * resources contained in this {@link LocallyInstalledPackage} and, by
	 * delegation, in its dependencies.
	 * 
	 * @return a {@link Loader} associated with this
	 *         {@link LocallyInstalledPackage}
	 */
	public Loader getLoader();

	/**
	 * Returns a number of {@link LocallyInstalledPackage}s that depend on this
	 * one
	 * 
	 * @return A number of {@link LocallyInstalledPackage}s that depend on this
	 *         one
	 */
	public int getReferenceCounter();

	/**
	 * Increases by one (increments) the reference counter of this
	 * {@link LocallyInstalledPackage}
	 */
	public void increaseReferenceCounter();

	/**
	 * Decreases by one (decrements) the reference counter of this
	 * {@link LocallyInstalledPackage}
	 */
	public void decreaseReferenceCounter();

	/**
	 * Returs a list of {@link LocallyInstalledPackage}s that this
	 * {@link LocallyInstalledPackage} depends on
	 * 
	 * @return A list of {@link LocallyInstalledPackage}s that this
	 *         {@link LocallyInstalledPackage} depends on
	 */
	public List<LocallyInstalledPackage> getDependencies();

	/**
	 * Returns a {@link PackageDescription} that corresponds to this
	 * {@link LocallyIntalledPackage}
	 * 
	 * @return A {@link PackageDescription} that corresponds to this
	 *         {@link LocallyIntalledPackage}
	 */
	public PackageDescription getPackageDescription();

	/**
	 * A marked {@link LocallyInstalledPackage} will never by garbage collected
	 * 
	 * @return true if this {@link LocallyInstalledPackage} is marked
	 */
	public boolean isMarked();

	/**
	 * Completely removes this {@link LocallyInstalledPackage} from the local
	 * filesysem, even if it is marked
	 */
	public void unInstall();

}
